@@ -1,6 +1,8 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 import Test.Hspec
 import Test.Hspec.QuickCheck (prop)
 import Test.QuickCheck.Monadic (assert, monadicIO, run)
@@ -32,6 +34,7 @@ import qualified Control.Concurrent.MVar as M
 import Control.Monad.Error (catchError, throwError, Error)
 import qualified Data.Map as Map
 import qualified Data.Conduit.Extra.ZipConduitSpec as ZipConduit
+import qualified Data.Conduit.StreamSpec as Stream
 
 (@=?) :: (Eq a, Show a) => a -> a -> IO ()
 (@=?) = flip shouldBe
@@ -219,8 +222,8 @@ main = hspec $ do
         it "map, left >+>" $ do
             x <- runResourceT $
                 CI.ConduitM
-                    (CI.unConduitM (CL.sourceList [1..10])
-                    CI.>+> CI.injectLeftovers (CI.unConduitM $ CL.map (* 2)))
+                    ((CI.unConduitM (CL.sourceList [1..10]) CI.Done
+                    CI.>+> CI.injectLeftovers (flip CI.unConduitM CI.Done $ CL.map (* 2))) >>=)
                     C.$$ CL.fold (+) 0
             x `shouldBe` 2 * sum [1..10 :: Int]
 
@@ -245,6 +248,21 @@ main = hspec $ do
                     C.=$ CL.consume
             x `shouldBe` DL.groupBy (==) input
 
+        it "groupOn1" $ do
+            let input = [1::Int, 1, 2, 3, 3, 3, 4, 5, 5]
+            x <- runResourceT $ CL.sourceList input
+                    C.$$ CL.groupOn1 id
+                    C.=$ CL.consume
+            x `shouldBe` [(1,[1]), (2, []), (3,[3,3]), (4,[]), (5, [5])]
+
+        it "groupOn1 (nondup begin/end)" $ do
+            let input = [1::Int, 2, 3, 3, 3, 4, 5]
+            x <- runResourceT $ CL.sourceList input
+                    C.$$ CL.groupOn1 id
+                    C.=$ CL.consume
+            x `shouldBe` [(1,[]), (2, []), (3,[3,3]), (4,[]), (5, [])]
+
+
         it "mapMaybe" $ do
             let input = [Just (1::Int), Nothing, Just 2, Nothing, Just 3]
             x <- runResourceT $ CL.sourceList input
@@ -444,8 +462,8 @@ main = hspec $ do
             ref <- I.newIORef []
             let add x = I.modifyIORef ref (x:)
                 adder' = CI.NeedInput (\a -> liftIO (add a) >> adder') return
-                adder = CI.ConduitM adder'
-                residue x = CI.ConduitM $ CI.Leftover (CI.Done ()) x
+                adder = CI.ConduitM (adder' >>=)
+                residue x = CI.ConduitM $ \rest -> CI.Leftover (rest ()) x
 
             _ <- C.yield 1 C.$$ adder
             x <- I.readIORef ref
@@ -509,12 +527,12 @@ main = hspec $ do
 
     describe "left/right identity" $ do
         it' "left identity" $ do
-            x <- CL.sourceList [1..10 :: Int] C.$$ CI.ConduitM CI.idP C.=$ CL.fold (+) 0
+            x <- CL.sourceList [1..10 :: Int] C.$$ CI.ConduitM (CI.idP >>=) C.=$ CL.fold (+) 0
             y <- CL.sourceList [1..10 :: Int] C.$$ CL.fold (+) 0
             x `shouldBe` y
         it' "right identity" $ do
-            x <- CI.runPipe $ mapM_ CI.yield [1..10 :: Int] CI.>+> (CI.injectLeftovers $ CI.unConduitM $ CL.fold (+) 0) CI.>+> CI.idP
-            y <- CI.runPipe $ mapM_ CI.yield [1..10 :: Int] CI.>+> (CI.injectLeftovers $ CI.unConduitM $ CL.fold (+) 0)
+            x <- CI.runPipe $ mapM_ CI.yield [1..10 :: Int] CI.>+> (CI.injectLeftovers $ flip CI.unConduitM CI.Done $ CL.fold (+) 0) CI.>+> CI.idP
+            y <- CI.runPipe $ mapM_ CI.yield [1..10 :: Int] CI.>+> (CI.injectLeftovers $ flip CI.unConduitM CI.Done $ CL.fold (+) 0)
             x `shouldBe` y
 
     describe "generalizing" $ do
@@ -546,6 +564,20 @@ main = hspec $ do
             res <- CL.iterate (+ 1) (1 :: Int) C.$$ CL.isolate 10 C.=$ CL.fold (+) 0
             res `shouldBe` sum [1..10]
 
+    prop "replicate" $ \cnt' -> do
+        let cnt = min cnt' 100
+        res <- CL.replicate cnt () C.$$ CL.consume
+        res `shouldBe` replicate cnt ()
+
+    prop "replicateM" $ \cnt' -> do
+        ref <- I.newIORef 0
+        let cnt = min cnt' 100
+        res <- CL.replicateM cnt (I.modifyIORef ref (+ 1)) C.$$ CL.consume
+        res `shouldBe` replicate cnt ()
+
+        ref' <- I.readIORef ref
+        ref' `shouldBe` (if cnt' <= 0 then 0 else cnt)
+
     describe "unwrapResumable" $ do
         it' "works" $ do
             ref <- I.newIORef (0 :: Int)
@@ -620,11 +652,11 @@ main = hspec $ do
     describe "injectLeftovers" $ do
         it "works" $ do
             let src = mapM_ CI.yield [1..10 :: Int]
-                conduit = CI.injectLeftovers $ CI.unConduitM $ C.awaitForever $ \i -> do
+                conduit = CI.injectLeftovers $ flip CI.unConduitM CI.Done $ C.awaitForever $ \i -> do
                     js <- CL.take 2
                     mapM_ C.leftover $ reverse js
                     C.yield i
-            res <- CI.ConduitM (src CI.>+> CI.injectLeftovers conduit) C.$$ CL.consume
+            res <- CI.ConduitM ((src CI.>+> CI.injectLeftovers conduit) >>=) C.$$ CL.consume
             res `shouldBe` [1..10]
     describe "up-upstream finalizers" $ do
         it "pipe" $ do
@@ -902,6 +934,27 @@ main = hspec $ do
                 return (x, y, z)
             res `shouldBe` (sum [1..5], ["hello"], [6..10])
 
+        it "fuseBothMaybe with no result" $ do
+            let src = mapM_ C.yield [1 :: Int ..]
+                sink = CL.isolate 5 C.=$= CL.fold (+) 0
+            (mup, down) <- C.runConduit $ C.fuseBothMaybe src sink
+            mup `shouldBe` (Nothing :: Maybe ())
+            down `shouldBe` sum [1..5]
+
+        it "fuseBothMaybe with result" $ do
+            let src = mapM_ C.yield [1 :: Int .. 5]
+                sink = CL.isolate 6 C.=$= CL.fold (+) 0
+            (mup, down) <- C.runConduit $ C.fuseBothMaybe src sink
+            mup `shouldBe` Just ()
+            down `shouldBe` sum [1..5]
+
+        it "fuseBothMaybe with almost result" $ do
+            let src = mapM_ C.yield [1 :: Int .. 5]
+                sink = CL.isolate 5 C.=$= CL.fold (+) 0
+            (mup, down) <- C.runConduit $ C.fuseBothMaybe src sink
+            mup `shouldBe` (Nothing :: Maybe ())
+            down `shouldBe` sum [1..5]
+
     describe "catching exceptions" $ do
         it "works" $ do
             let src = do
@@ -914,6 +967,7 @@ main = hspec $ do
             res `shouldBe` [1, 3]
 
     ZipConduit.spec
+    Stream.spec
 
 it' :: String -> IO () -> Spec
 it' = it
